<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:xlf="urn:oasis:names:tc:xliff:document:1.2"
	xmlns:xf="xl:ffer"
	exclude-result-prefixes="xlf">

	<xsl:param name="transUnitCheck" select="false()"/>

	<xsl:key name="files-by-source-language" match="xlf:file" use="@source-language"/>

	<xsl:key name="files-by-target-language" match="xlf:file" use="@target-language"/>

	<xsl:key name="files-by-original" match="xlf:file" use="@original"/>

	<xsl:key name="files-by-datatype" match="xlf:file" use="@datatype"/>

	<xsl:variable name="fileCount" select="count(/xf:collection/xlf:xliff/xlf:file)"/>

	<xsl:template match="/">
		<xf:consistency>
			<xsl:if test="count(/xf:collection/xlf:xliff/xlf:file[@source-language and not(@target-language)]) &gt; 1">
				<xf:problem xf:level="error" xf:type="MultipleSourceLangFiles" xf:group="overall-source-language">
					<xsl:for-each select="/xf:collection/xlf:xliff/xlf:file[@source-language and not(@target-language)]">
						<xsl:call-template name="pointer">
							<xsl:with-param name="document" select="../@xf:document"/>
							<xsl:with-param name="filePosition" select="position()"/>
							<xsl:with-param name="filePredicate">
								<xsl:text>@source-language=&apos;</xsl:text>
								<xsl:value-of select="@source-language"/>
								<xsl:text>&apos;</xsl:text>
							</xsl:with-param>
						</xsl:call-template>
					</xsl:for-each>
				</xf:problem>
			</xsl:if>
			<xsl:if test="count(/xf:collection/xlf:xliff/xlf:file[@source-language and not(@target-language)]) = 0">
				<xf:problem xf:level="error" xf:type="NoSourceLangFile" xf:group="overall-source-language"/>
			</xsl:if>
			<xsl:if test="count(/xf:collection/xlf:xliff/xlf:file[@target-language]) = 0">
				<xf:problem xf:level="warning" xf:type="NoTargetLangFile" xf:group="overall-target-language"/>
			</xsl:if>
			<xsl:apply-templates select="xf:collection/xlf:xliff"/>
		</xf:consistency>
	</xsl:template>

	<xsl:template match="xlf:xliff">
		<xsl:variable name="document" select="@xf:document"/>
		<xsl:variable name="docBaseName" select="@xf:docBaseName"/>
		<xsl:variable name="docPrefix" select="@xf:docPrefix"/>

		<xsl:if test="count(xlf:file) &gt; 1">
			<xf:problem xf:level="warning" xf:type="MultipleFilesInDoc" xf:group="name-scheme" xf:document="{$document}">
				<xsl:for-each select="xlf:file">
					<xsl:call-template name="pointer">
					<xsl:with-param name="document" select="$document"/>
					<xsl:with-param name="filePosition" select="position()"/>
				</xsl:call-template>
				</xsl:for-each>
			</xf:problem>
		</xsl:if>

		<xsl:apply-templates select="xlf:file">
			<xsl:with-param name="document" select="$document"/>
			<xsl:with-param name="docBaseName" select="$docBaseName"/>
			<xsl:with-param name="docPrefix" select="$docPrefix"/>
		</xsl:apply-templates>

		<xsl:apply-templates select="xf:parseError">
			<xsl:with-param name="document" select="$document"/>
		</xsl:apply-templates>

	</xsl:template>

	<xsl:template match="xlf:file">
		<xsl:param name="document"/>
		<xsl:param name="docBaseName"/>
		<xsl:param name="docPrefix"/>
		<xsl:variable name="fileType">
			<xsl:choose>
				<xsl:when test="@source-language and not(@target-language)">source</xsl:when>
				<xsl:when test="@target-language">target</xsl:when>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="langCode">
			<xsl:choose>
				<xsl:when test="$fileType = 'source'">
					<xsl:value-of select="@source-language"/>
				</xsl:when>
				<xsl:when test="$fileType = 'target'">
					<xsl:value-of select="@target-language"/>
				</xsl:when>
			</xsl:choose>
		</xsl:variable>
		<xsl:choose>
			<xsl:when test="not(@source-language)">
				<xf:problem xf:level="error" xf:type="NoSourceLang" xf:group="@source-language" xf:document="{$document}">
					<xsl:call-template name="pointer">
						<xsl:with-param name="document" select="$document"/>
						<xsl:with-param name="filePosition" select="position()"/>
					</xsl:call-template>
				</xf:problem>
			</xsl:when>
			<xsl:when test="count(key('files-by-source-language', @source-language)) != $fileCount">
				<xf:problem xf:level="warning" xf:type="DifferentSourceLangs" xf:group="overall-source-language">
					<xsl:call-template name="pointer">
						<xsl:with-param name="document" select="$document"/>
						<xsl:with-param name="filePosition" select="position()"/>
						<xsl:with-param name="filePredicate">
							<xsl:text>@source-language=&apos;</xsl:text>
							<xsl:value-of select="@source-language"/>
							<xsl:text>&apos;</xsl:text>
						</xsl:with-param>
					</xsl:call-template>
				</xf:problem>
			</xsl:when>
		</xsl:choose>
		<xsl:if test="@source-language and not(@target-language) and $docPrefix">
			<xf:problem xf:level="warning" xf:type="SourceLangNameSchemeMismatch" xf:group="name-scheme" xf:document="{$document}">
				<xsl:call-template name="pointer">
					<xsl:with-param name="document" select="$document"/>
					<xsl:with-param name="filePosition" select="position()"/>
					<xsl:with-param name="filePredicate">
						<xsl:text>@source-language=&apos;</xsl:text>
						<xsl:value-of select="@source-language"/>
						<xsl:text>&apos;</xsl:text>
					</xsl:with-param>
				</xsl:call-template>
			</xf:problem>
		</xsl:if>
		<xsl:if test="@target-language and count(key('files-by-target-language', @target-language)) > 1">
			<xf:problem xf:level="error" xf:type="SameTargetLangFiles" xf:group="overall-target-language">
				<xsl:call-template name="pointer">
					<xsl:with-param name="document" select="$document"/>
					<xsl:with-param name="filePosition" select="position()"/>
					<xsl:with-param name="filePredicate">
						<xsl:text>@target-language=&apos;</xsl:text>
						<xsl:value-of select="@target-language"/>
						<xsl:text>&apos;</xsl:text>
					</xsl:with-param>
				</xsl:call-template>
			</xf:problem>
		</xsl:if>
		<xsl:if test="@target-language and @target-language != $docPrefix">
			<xf:problem xf:level="warning" xf:type="TargetLangNameSchemeMismatch" xf:group="name-scheme" xf:document="{$document}">
				<xsl:call-template name="pointer">
					<xsl:with-param name="document" select="$document"/>
					<xsl:with-param name="filePosition" select="position()"/>
					<xsl:with-param name="filePredicate">
						<xsl:text>@target-language=&apos;</xsl:text>
						<xsl:value-of select="@target-language"/>
						<xsl:text>&apos;</xsl:text>
					</xsl:with-param>
				</xsl:call-template>
			</xf:problem>
		</xsl:if>
		<xsl:choose>
			<xsl:when test="not(@original)">
				<xf:problem xf:level="warning" xf:type="NoOriginal" xf:group="@original" xf:document="{$document}">
					<xsl:call-template name="pointer">
						<xsl:with-param name="document" select="$document"/>
						<xsl:with-param name="filePosition" select="position()"/>
					</xsl:call-template>
				</xf:problem>
			</xsl:when>
			<xsl:when test="count(key('files-by-original', @original)) != $fileCount">
				<xf:problem xf:level="warning" xf:type="DifferentOriginals" xf:group="overall-original">
					<xsl:call-template name="pointer">
						<xsl:with-param name="document" select="$document"/>
						<xsl:with-param name="filePosition" select="position()"/>
						<xsl:with-param name="filePredicate">
							<xsl:text>@original=&apos;</xsl:text>
							<xsl:value-of select="@original"/>
							<xsl:text>&apos;</xsl:text>
						</xsl:with-param>
					</xsl:call-template>
				</xf:problem>
			</xsl:when>
		</xsl:choose>
		<xsl:choose>
			<xsl:when test="not(@datatype)">
				<xf:problem xf:level="warning" xf:type="NoDatatype" xf:group="@datatype" xf:document="{$document}">
					<xsl:call-template name="pointer">
						<xsl:with-param name="document" select="$document"/>
						<xsl:with-param name="filePosition" select="position()"/>
					</xsl:call-template>
				</xf:problem>
			</xsl:when>
			<xsl:when test="count(key('files-by-datatype', @datatype)) != $fileCount">
				<xf:problem xf:level="warning" xf:type="DifferentDatatypes" xf:group="overall-datatype">
					<xsl:call-template name="pointer">
						<xsl:with-param name="document" select="$document"/>
						<xsl:with-param name="filePosition" select="position()"/>
						<xsl:with-param name="filePredicate">
							<xsl:text>@datatype=&apos;</xsl:text>
							<xsl:value-of select="@datatype"/>
							<xsl:text>&apos;</xsl:text>
						</xsl:with-param>
					</xsl:call-template>
				</xf:problem>
			</xsl:when>
		</xsl:choose>
		<xsl:if test="$transUnitCheck">
			<xsl:apply-templates select="xlf:body/xlf:trans-unit">
				<xsl:with-param name="document" select="$document"/>
				<xsl:with-param name="filePosition" select="position()"/>
				<xsl:with-param name="fileType" select="$fileType"/>
				<xsl:with-param name="langCode" select="$langCode"/>
			</xsl:apply-templates>
		</xsl:if>
	</xsl:template>

	<xsl:template match="xlf:trans-unit">
		<xsl:param name="document"/>
		<xsl:param name="filePosition"/>
		<xsl:param name="fileType"/>
		<xsl:param name="langCode"/>
		<xsl:variable name="transUnitId" select="@id"/>
		<xsl:choose>
			<xsl:when test="$fileType = 'source'">
				<xsl:apply-templates select="/xf:collection/xlf:xliff/xlf:file[@target-language]" mode="trans-unit-check">
					<xsl:with-param name="document" select="$document"/>
					<xsl:with-param name="filePosition" select="$filePosition"/>
					<xsl:with-param name="fileType" select="$fileType"/>
					<xsl:with-param name="langCode" select="$langCode"/>
					<xsl:with-param name="transUnitId" select="$transUnitId"/>
				</xsl:apply-templates>
			</xsl:when>
			<xsl:when test="$fileType = 'target'">
				<xsl:apply-templates select="/xf:collection/xlf:xliff/xlf:file[not(@target-language)]" mode="trans-unit-check">
					<xsl:with-param name="document" select="$document"/>
					<xsl:with-param name="filePosition" select="$filePosition"/>
					<xsl:with-param name="fileType" select="$fileType"/>
					<xsl:with-param name="langCode" select="$langCode"/>
					<xsl:with-param name="transUnitId" select="$transUnitId"/>
					<xsl:with-param name="source" select="normalize-space(xlf:source)"/>
					<xsl:with-param name="target" select="normalize-space(xlf:target)"/>
				</xsl:apply-templates>
			</xsl:when>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="xlf:file" mode="trans-unit-check">
		<xsl:param name="document"/>
		<xsl:param name="filePosition"/>
		<xsl:param name="fileType"/>
		<xsl:param name="langCode"/>
		<xsl:param name="transUnitId"/>
		<xsl:param name="source" select="false()"/>
		<xsl:param name="target" select="false()"/>
		<xsl:variable name="warningType">
			<xsl:choose>
				<xsl:when test="$fileType = 'source'">TransUnitMissingInTargetLangFile</xsl:when>
				<xsl:when test="$fileType = 'target'">TransUnitMissingInSourceLangFile</xsl:when>
			</xsl:choose>
		</xsl:variable>
		<xsl:choose>
			<xsl:when test="not(xlf:body/xlf:trans-unit[@id = $transUnitId])">
				<xf:problem xf:level="warning" xf:type="{$warningType}" xf:group="trans-unit" xf:document="{$document}">
					<xsl:call-template name="pointer">
						<xsl:with-param name="document">
							<xsl:choose>
								<xsl:when test="$fileType = 'source'">
									<xsl:value-of select="../@xf:document"/>
								</xsl:when>
								<xsl:when test="$fileType = 'target'">
									<xsl:value-of select="$document"/>
								</xsl:when>
							</xsl:choose>
						</xsl:with-param>
						<xsl:with-param name="filePosition">
							<xsl:choose>
								<xsl:when test="$fileType = 'source'">
									<xsl:value-of select="position()"/>
								</xsl:when>
								<xsl:when test="$fileType = 'target'">
									<xsl:value-of select="$filePosition"/>
								</xsl:when>
							</xsl:choose>
						</xsl:with-param>
						<xsl:with-param name="filePredicate">
							<xsl:text>@target-language=&apos;</xsl:text>
							<xsl:choose>
								<xsl:when test="$fileType = 'source'">
									<xsl:value-of select="@target-language"/>
								</xsl:when>
								<xsl:when test="$fileType = 'target'">
									<xsl:value-of select="$langCode"/>
								</xsl:when>
							</xsl:choose>
							<xsl:text>&apos;</xsl:text>
						</xsl:with-param>
						<xsl:with-param name="transUnitId" select="$transUnitId"/>
					</xsl:call-template>
				</xf:problem>
			</xsl:when>
			<xsl:when test="$fileType = 'target'">
				<xsl:if test="$source != normalize-space(/xf:collection/xlf:xliff/xlf:file[@source-language and not(@target-language)]/xlf:body/xlf:trans-unit[@id = $transUnitId]/xlf:source)">
					<xf:problem xf:level="warning" xf:type="TransUnitSourceMismatch" xf:group="trans-unit" xf:document="{$document}">
						<xsl:call-template name="pointer">
							<xsl:with-param name="document" select="$document"/>
							<xsl:with-param name="filePosition" select="$filePosition"/>
							<xsl:with-param name="filePredicate">
								<xsl:text>@target-language=&apos;</xsl:text>
								<xsl:value-of select="$langCode"/>
								<xsl:text>&apos;</xsl:text>
							</xsl:with-param>
							<xsl:with-param name="transUnitId" select="$transUnitId"/>
						</xsl:call-template>
					</xf:problem>
				</xsl:if>
				<xsl:if test="boolean($target) = false()">
					<xf:problem xf:level="warning" xf:type="TransUnitMissingTarget" xf:group="trans-unit" xf:document="{$document}">
						<xsl:call-template name="pointer">
							<xsl:with-param name="document" select="$document"/>
							<xsl:with-param name="filePosition" select="$filePosition"/>
							<xsl:with-param name="filePredicate">
								<xsl:text>@target-language=&apos;</xsl:text>
								<xsl:value-of select="$langCode"/>
								<xsl:text>&apos;</xsl:text>
							</xsl:with-param>
							<xsl:with-param name="transUnitId" select="$transUnitId"/>
						</xsl:call-template>
					</xf:problem>
				</xsl:if>
			</xsl:when>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="pointer">
		<xsl:param name="document"/>
		<xsl:param name="filePosition"/>
		<xsl:param name="filePredicate" select="false()"/>
		<xsl:param name="transUnitId" select="false()"/>
		<xf:pointer>
			<xsl:text>doc(&apos;</xsl:text>
			<xsl:value-of select="$document"/>
			<xsl:text>&apos;)/xliff/file[</xsl:text>
			<xsl:value-of select="$filePosition"/>
			<xsl:text>]</xsl:text>
			<xsl:if test="$filePredicate">
				<xsl:text>[</xsl:text>
				<xsl:value-of select="$filePredicate"/>
				<xsl:text>]</xsl:text>
			</xsl:if>
			<xsl:if test="$transUnitId">
				<xsl:text>/body/trans-unit[@id=&apos;</xsl:text>
				<xsl:value-of select="$transUnitId"/>
				<xsl:text>&apos;]</xsl:text>
			</xsl:if>
		</xf:pointer>
	</xsl:template>

	<xsl:template match="xf:parseError">
		<xsl:param name="document"/>
		<xf:invalidXliffWarning xf:document="{$document}">
			<xsl:apply-templates select="xf:*"/>
		</xf:invalidXliffWarning>
	</xsl:template>

	<xsl:template match="xf:*">
		<xsl:element name="{name()}" namespace="xl:ffer">
			<xsl:apply-templates/>
		</xsl:element>
	</xsl:template>

</xsl:stylesheet>