<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns="http://ns.adobe.com/GenericCustomNamespace/1.0/"
	xmlns:xlf="urn:oasis:names:tc:xliff:document:1.2"
	exclude-result-prefixes="xlf">

	<xsl:output indent="yes"
		doctype-public="-//W3C//DTD SVG 20001102//EN"
		doctype-system="http://www.w3.org/TR/2000/CR-SVG-20001102/DTD/svg-20001102.dtd"/>

	<!-- Stylesheet for merged xliff -->

	<xsl:param name="sourceLanguageDataSet" select="false()"/>
	<xsl:param name="allTargetLanguageDataSets" select="false()"/>
	<xsl:param name="targetLanguageDataSets" select="false()"/>
	<xsl:param name="addVisibilityVariables" select="false()"/>

	<xsl:variable name="sourceLanguage" select="/xlf:xliff/xlf:file[not(@target-language)]/@source-language"/>
	<xsl:variable name="targetLangDataSetsString">
		<xsl:choose>
			<xsl:when test="$allTargetLanguageDataSets">
				<xsl:for-each select="/xlf:xliff/xlf:file/@target-language">
					<xsl:value-of select="concat('|', .)"/>
				</xsl:for-each>
			</xsl:when>
			<xsl:when test="$targetLanguageDataSets">
				<xsl:value-of select="concat('|', $targetLanguageDataSets)"/>
			</xsl:when>
		</xsl:choose>
	</xsl:variable>
	<xsl:variable name="languages" select="concat($sourceLanguage, $targetLangDataSetsString)"/>
	<xsl:variable name="targetLangDataSets">
		<xsl:if test="$targetLangDataSetsString">
			<xsl:value-of select="substring($targetLangDataSetsString, 2)"/>
		</xsl:if>
	</xsl:variable>

	<xsl:template match="/">
		<xsl:element name="svg" namespace="">
			<variableSets xmlns="http://ns.adobe.com/Variables/1.0/">
				<variableSet locked="none" varSetName="binding1">
					<variables>
						<xsl:apply-templates select="/xlf:xliff/xlf:file[not(@target-language)]/xlf:body/xlf:trans-unit"/>
						<xsl:if test="$addVisibilityVariables">
							<xsl:call-template name="visibilityVariables"/>
						</xsl:if>
					</variables>
					<v:sampleDataSets xmlns:v="http://ns.adobe.com/Variables/1.0/" xmlns="http://ns.adobe.com/GenericCustomNamespace/1.0/">
						<xsl:call-template name="sourceDataSet"/>
						<xsl:call-template name="targetDataSets"/>
					</v:sampleDataSets>
				</variableSet>
			</variableSets>
		</xsl:element>
	</xsl:template>

	<xsl:template name="visibilityVariables">
		<xsl:param name="langs" select="$languages"/>
		<xsl:variable name="lang">
			<xsl:choose>
				<xsl:when test="contains($langs, '|')">
					<xsl:value-of select="normalize-space(substring-before($langs, '|'))"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="normalize-space($langs)"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="langCode">
			<xsl:choose>
				<xsl:when test="contains($lang, '·')">
					<xsl:value-of select="normalize-space(substring-after($lang, '·'))"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="normalize-space($lang)"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<variable varName="visible_in_{$langCode}" trait="visibility" category="http://ns.adobe.com/Variables/1.0/" xmlns="http://ns.adobe.com/Variables/1.0/"/>
		<xsl:if test="contains($langs, '|')">
			<xsl:call-template name="visibilityVariables">
				<xsl:with-param name="langs" select="substring-after($langs, '|')"/>
			</xsl:call-template>
		</xsl:if>
	</xsl:template>

	<xsl:template name="sourceDataSet">
		<xsl:variable name="sourceDataSetName">
			<xsl:choose>
				<xsl:when test="$sourceLanguageDataSet">
					<xsl:value-of select="normalize-space($sourceLanguageDataSet)"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="$sourceLanguage"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<v:sampleDataSet dataSetName="{$sourceDataSetName}" xmlns:v="http://ns.adobe.com/Variables/1.0/">
			<xsl:apply-templates select="/xlf:xliff/xlf:file[not(@target-language)]/xlf:body/xlf:trans-unit/xlf:source"/>
			<xsl:if test="$addVisibilityVariables">
				<xsl:call-template name="dataSetVisibilityVariables">
					<xsl:with-param name="activeLangCode" select="$sourceLanguage"/>
				</xsl:call-template>
			</xsl:if>
		</v:sampleDataSet>
	</xsl:template>

	<xsl:template name="targetDataSets">
		<xsl:param name="set" select="$targetLangDataSets"/>
		<xsl:variable name="activeSet">
			<xsl:choose>
				<xsl:when test="contains($set, '|')">
					<xsl:value-of select="normalize-space(substring-before($set,'|'))"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="normalize-space($set)"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="targetDataSetName">
			<xsl:choose>
				<xsl:when test="contains($activeSet, '·')">
					<xsl:value-of select="normalize-space(substring-before($activeSet, '·'))"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="normalize-space($activeSet)"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="targetLanguage">
			<xsl:choose>
				<xsl:when test="contains($activeSet, '·')">
					<xsl:value-of select="normalize-space(substring-after($activeSet, '·'))"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="normalize-space($activeSet)"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<v:sampleDataSet dataSetName="{$targetDataSetName}" xmlns:v="http://ns.adobe.com/Variables/1.0/">
			<xsl:apply-templates
				select="/xlf:xliff/xlf:file[@target-language = $targetLanguage]/xlf:body/xlf:trans-unit/xlf:target"/>
			<xsl:if test="$addVisibilityVariables">
				<xsl:call-template name="dataSetVisibilityVariables">
					<xsl:with-param name="activeLangCode" select="$targetLanguage"/>
				</xsl:call-template>
			</xsl:if>
		</v:sampleDataSet>
		<xsl:if test="contains($set, '|')">
			<xsl:call-template name="targetDataSets">
				<xsl:with-param name="set" select="substring-after($set, '|')"/>
			</xsl:call-template>
		</xsl:if>
	</xsl:template>

	<xsl:template name="dataSetVisibilityVariables">
		<xsl:param name="activeLangCode"/>
		<xsl:param name="langs" select="$languages"/>
		<xsl:variable name="lang">
			<xsl:choose>
				<xsl:when test="contains($langs, '|')">
					<xsl:value-of select="normalize-space(substring-before($langs, '|'))"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="normalize-space($langs)"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="langCode">
			<xsl:choose>
				<xsl:when test="contains($lang, '·')">
					<xsl:value-of select="normalize-space(substring-after($lang, '·'))"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="normalize-space($lang)"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:element name="visible_in_{$langCode}">
			<xsl:choose>
				<xsl:when test="$activeLangCode = $langCode">
					<xsl:text>true</xsl:text>
				</xsl:when>
				<xsl:otherwise>
					<xsl:text>false</xsl:text>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:element>
		<xsl:if test="contains($langs, '|')">
			<xsl:call-template name="dataSetVisibilityVariables">
				<xsl:with-param name="activeLangCode" select="$activeLangCode"/>
				<xsl:with-param name="langs" select="substring-after($langs, '|')"/>
			</xsl:call-template>
		</xsl:if>
	</xsl:template>

	<xsl:template match="xlf:trans-unit">
		<variable trait="textcontent" varName="{@id}" category="http://ns.adobe.com/Flows/1.0/" xmlns="http://ns.adobe.com/Variables/1.0/"/>
	</xsl:template>

	<xsl:template match="xlf:source|xlf:target">
		<xsl:element name="{../@id}">
			<xsl:apply-templates select="text()"/>
		</xsl:element>
	</xsl:template>

	<xsl:template match="text()">
		<p><xsl:value-of select="."/></p>
	</xsl:template>
</xsl:stylesheet>